==> gcsio-2.1.6-sources.jar: com/google/cloud/hadoop/gcsio/GoogleCloudStorageWriteChannel.java <==
/*
 * Copyright 2013 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.cloud.hadoop.gcsio;

import com.google.api.client.http.InputStreamContent;
import com.google.api.services.storage.Storage;
import com.google.api.services.storage.Storage.Objects.Insert;
import com.google.api.services.storage.model.StorageObject;
import com.google.cloud.hadoop.util.AbstractGoogleAsyncWriteChannel;
import com.google.cloud.hadoop.util.ApiErrorExtractor;
import com.google.cloud.hadoop.util.AsyncWriteChannelOptions;
import com.google.cloud.hadoop.util.ClientRequestHelper;
import com.google.cloud.hadoop.util.LoggingMediaHttpUploaderProgressListener;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ExecutorService;

/** Implements WritableByteChannel to provide write access to GCS. */
public class GoogleCloudStorageWriteChannel
    extends AbstractGoogleAsyncWriteChannel<Insert, StorageObject>
    implements GoogleCloudStorageItemInfo.Provider {

  private static final long MIN_LOGGING_INTERVAL_MS = 60000L;

  private final Storage gcs;
  private final String bucketName;
  private final String objectName;
  private final String contentEncoding;
  private final String kmsKeyName;
  private final ObjectWriteConditions writeConditions;
  private final Map<String, String> metadata;

  private GoogleCloudStorageItemInfo completedItemInfo = null;

  /**
   * Constructs an instance of GoogleCloudStorageWriteChannel.
   *
   * @param uploadThreadPool thread pool to use for running the upload operation
   * @param gcs storage object instance
   * @param requestHelper a ClientRequestHelper to set extra headers
   * @param bucketName name of the bucket to create object in
   * @param objectName name of the object to create
   * @param writeConditions conditions on which write should be allowed to continue
   * @param objectMetadata metadata to apply to the newly created object
   */
  @Deprecated
  public GoogleCloudStorageWriteChannel(
      ExecutorService uploadThreadPool,
      Storage gcs,
      ClientRequestHelper<StorageObject> requestHelper,
      String bucketName,
      String objectName,
      AsyncWriteChannelOptions options,
      ObjectWriteConditions writeConditions,
      Map<String, String> objectMetadata) {
    this(
        uploadThreadPool,
        gcs,
        requestHelper,
        bucketName,
        objectName,
        /* contentType= */ null,
        /* kmsKeyName= */ null,
        options,
        writeConditions,
        objectMetadata);
  }

  /**
   * Constructs an instance of GoogleCloudStorageWriteChannel.
   *
   * @param uploadThreadPool thread pool to use for running the upload operation
   * @param gcs storage object instance
   * @param requestHelper a ClientRequestHelper to set extra headers
   * @param bucketName name of the bucket to create object in
   * @param objectName name of the object to create
   * @param writeConditions conditions on which write should be allowed to continue
   * @param objectMetadata metadata to apply to the newly created object
   * @param contentType content type
   */
  @Deprecated
  public GoogleCloudStorageWriteChannel(
      ExecutorService uploadThreadPool,
      Storage gcs,
      ClientRequestHelper<StorageObject> requestHelper,
      String bucketName,
      String objectName,
      AsyncWriteChannelOptions options,
      ObjectWriteConditions writeConditions,
      Map<String, String> objectMetadata,
      String contentType) {
    this(
        uploadThreadPool,
        gcs,
        requestHelper,
        bucketName,
        objectName,
        contentType,
        /* kmsKeyName= */ null,
        options,
        writeConditions,
        objectMetadata);
  }

  /**
   * Constructs an instance of GoogleCloudStorageWriteChannel.
   *
   * @param uploadThreadPool thread pool to use for running the upload operation
   * @param gcs storage object instance
   * @param requestHelper a ClientRequestHelper to set extra headers
   * @param bucketName name of the bucket to create object in
   * @param objectName name of the object to create
   * @param contentType content type
   * @param kmsKeyName Name of Cloud KMS key to use to encrypt the newly created object
   * @param writeConditions conditions on which write should be allowed to continue
   * @param objectMetadata metadata to apply to the newly created object
   */
  @Deprecated
  public GoogleCloudStorageWriteChannel(
      ExecutorService uploadThreadPool,
      Storage gcs,
      ClientRequestHelper<StorageObject> requestHelper,
      String bucketName,
      String objectName,
      String contentType,
      String kmsKeyName,
      AsyncWriteChannelOptions options,
      ObjectWriteConditions writeConditions,
      Map<String, String> objectMetadata) {
    this(
        uploadThreadPool,
        gcs,
        requestHelper,
        bucketName,
        objectName,
        contentType,
        /* contentEncoding= */ null,
        kmsKeyName,
        options,
        writeConditions,
        objectMetadata);
  }
  /**
   * Constructs an instance of GoogleCloudStorageWriteChannel.
   *
   * @param uploadThreadPool thread pool to use for running the upload operation
   * @param gcs storage object instance
   * @param requestHelper a ClientRequestHelper to set extra headers
   * @param bucketName name of the bucket to create object in
   * @param objectName name of the object to create
   * @param contentType content type
   * @param contentEncoding content encoding
   * @param kmsKeyName Name of Cloud KMS key to use to encrypt the newly created object
   * @param writeConditions conditions on which write should be allowed to continue
   * @param objectMetadata metadata to apply to the newly created object
   */
  public GoogleCloudStorageWriteChannel(
      ExecutorService uploadThreadPool,
      Storage gcs,
      ClientRequestHelper<StorageObject> requestHelper,
      String bucketName,
      String objectName,
      String contentType,
      String contentEncoding,
      String kmsKeyName,
      AsyncWriteChannelOptions options,
      ObjectWriteConditions writeConditions,
      Map<String, String> objectMetadata) {
    super(uploadThreadPool, options);
    this.gcs = gcs;
    this.setClientRequestHelper(requestHelper);
    this.bucketName = bucketName;
    this.objectName = objectName;
    if (contentType != null) {
      setContentType(contentType);
    }
    this.contentEncoding = contentEncoding;
    this.kmsKeyName = kmsKeyName;
    this.writeConditions = writeConditions;
    this.metadata = objectMetadata;
  }

  @Override
  public Insert createRequest(InputStreamContent inputStream) throws IOException {
    // Create object with the given name and metadata.
    StorageObject object =
        new StorageObject()
            .setContentEncoding(contentEncoding)
            .setMetadata(metadata)
            .setName(objectName);

    Insert insert = gcs.objects().insert(bucketName, object, inputStream);
    writeConditions.apply(insert);
    if (insert.getMediaHttpUploader() != null) {
      insert.getMediaHttpUploader().setDirectUploadEnabled(isDirectUploadEnabled());
      insert.getMediaHttpUploader().setProgressListener(
        new LoggingMediaHttpUploaderProgressListener(this.objectName, MIN_LOGGING_INTERVAL_MS));
    }
    insert.setName(objectName);
    if (kmsKeyName != null) {
      insert.setKmsKeyName(kmsKeyName);
    }
    return insert;
  }

  @Override
  public void handleResponse(StorageObject response) {
    StorageResourceId resourceId = new StorageResourceId(bucketName, objectName);
    this.completedItemInfo =
        writeConditions.getIgnoreGenerationMismatch()
            ? GoogleCloudStorageItemInfo.createNotFound(resourceId)
            : GoogleCloudStorageImpl.createItemInfoForStorageObject(resourceId, response);
  }

  @Override
  public StorageObject createResponseFromException(IOException e) {
    if (writeConditions.getIgnoreGenerationMismatch()
        && ApiErrorExtractor.INSTANCE.preconditionNotMet(e)) {
      logger.atWarning().withCause(e).log(
          "412 Precondition failure was ignored for resource '%s'", getResourceString());
      return new StorageObject().setBucket(bucketName).setName(objectName);
    }
    return super.createResponseFromException(e);
  }

  @Override
  protected String getResourceString() {
    return new StorageResourceId(bucketName, objectName).toString();
  }

  /**
   * Returns non-null only if close() has been called and the underlying object has been
   * successfully committed.
   */
  @Override
  public GoogleCloudStorageItemInfo getItemInfo() {
    return this.completedItemInfo;
  }
}

==> gcsio-2.2.4-sources.jar: com/google/cloud/hadoop/gcsio/GoogleCloudStorageWriteChannel.java <==
/*
 * Copyright 2013 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.cloud.hadoop.gcsio;

import static com.google.cloud.hadoop.gcsio.GoogleCloudStorageImpl.encodeMetadata;

import com.google.api.client.http.InputStreamContent;
import com.google.api.services.storage.Storage;
import com.google.api.services.storage.Storage.Objects.Insert;
import com.google.api.services.storage.model.StorageObject;
import com.google.cloud.hadoop.util.AbstractGoogleAsyncWriteChannel;
import com.google.cloud.hadoop.util.AsyncWriteChannelOptions;
import com.google.cloud.hadoop.util.ClientRequestHelper;
import com.google.cloud.hadoop.util.LoggingMediaHttpUploaderProgressListener;
import java.io.IOException;
import java.util.concurrent.ExecutorService;

/** Implements WritableByteChannel to provide write access to GCS. */
public class GoogleCloudStorageWriteChannel
    extends AbstractGoogleAsyncWriteChannel<Insert, StorageObject>
    implements GoogleCloudStorageItemInfo.Provider {

  private static final long MIN_LOGGING_INTERVAL_MS = 60000L;

  private final Storage gcs;
  private final StorageResourceId resourceId;
  private final CreateObjectOptions createOptions;
  private final ObjectWriteConditions writeConditions;

  private GoogleCloudStorageItemInfo completedItemInfo = null;

  /**
   * Constructs an instance of GoogleCloudStorageWriteChannel.
   *
   * @param gcs storage object instance
   * @param uploadThreadPool thread pool to use for running the upload operation
   * @param requestHelper a ClientRequestHelper to set extra headers
   * @param channelOptions write channel options
   * @param resourceId object to create
   * @param createOptions object creation options
   * @param writeConditions conditions on which write should be allowed to continue
   */
  public GoogleCloudStorageWriteChannel(
      Storage gcs,
      ClientRequestHelper<StorageObject> requestHelper,
      ExecutorService uploadThreadPool,
      AsyncWriteChannelOptions channelOptions,
      StorageResourceId resourceId,
      CreateObjectOptions createOptions,
      ObjectWriteConditions writeConditions) {
    super(requestHelper, uploadThreadPool, channelOptions);
    this.gcs = gcs;
    this.resourceId = resourceId;
    this.createOptions = createOptions;
    this.writeConditions = writeConditions;
  }

  @Override
  public Insert createRequest(InputStreamContent inputStream) throws IOException {
    // Create object with the given name and metadata.
    StorageObject object =
        new StorageObject()
            .setContentEncoding(createOptions.getContentEncoding())
            .setMetadata(encodeMetadata(createOptions.getMetadata()))
            .setName(resourceId.getObjectName());

    Insert insert =
        gcs.objects()
            .insert(resourceId.getBucketName(), object, inputStream)
            .setName(resourceId.getObjectName())
            .setKmsKeyName(createOptions.getKmsKeyName());
    writeConditions.apply(insert);
    if (insert.getMediaHttpUploader() != null) {
      insert
          .getMediaHttpUploader()
          .setDirectUploadEnabled(isDirectUploadEnabled())
          .setProgressListener(
              new LoggingMediaHttpUploaderProgressListener(
                  resourceId.getObjectName(), MIN_LOGGING_INTERVAL_MS));
    }
    return insert;
  }

  @Override
  public void handleResponse(StorageObject response) {
    this.completedItemInfo =
        GoogleCloudStorageImpl.createItemInfoForStorageObject(resourceId, response);
  }

  @Override
  protected String getContentType() {
    return createOptions.getContentType();
  }

  @Override
  protected String getResourceString() {
    return resourceId.toString();
  }

  /**
   * Returns non-null only if close() has been called and the underlying object has been
   * successfully committed.
   */
  @Override
  public GoogleCloudStorageItemInfo getItemInfo() {
    return completedItemInfo;
  }
}
